@@ -313,19 +313,18 @@ void nvidia_intr(void *xsc)
     }
 }
 
-int nvidia_get_card_info(struct nv_ioctl_card_info *ci)
+int nvidia_get_card_info(void *args, int size)
 {
+    struct nv_ioctl_card_info *ci;
+    struct nv_ioctl_rm_api_old_version *av;
     unsigned int i;
     struct nvidia_softc *sc;
     nv_state_t *nv;
 
-    /*
-     * Clients supporting versioning will pass version magic in the first
-     * card information field.
-     */
-    struct nv_ioctl_rm_api_old_version *av = (void *) ci;
-    int status = 0;
+    if (size < (sizeof(*ci) * NV_MAX_DEVICES))
+        return EINVAL;
 
+    av = args;
     switch (av->magic) {
         case NV_RM_API_OLD_VERSION_MAGIC_OVERRIDE_REQ:
         case NV_RM_API_OLD_VERSION_MAGIC_LAX_REQ:
@@ -341,9 +340,7 @@ int nvidia_get_card_info(struct nv_ioctl_card_info *ci)
                       "NVRM: components have the same version.\n",
                       av->major, av->minor, av->patch,
                       NV_VERSION_STRING);
-            status = EINVAL;
-            break;
-
+            return EINVAL;
         case NV_RM_API_OLD_VERSION_MAGIC_IGNORE:
             /*
              * the client is telling us to ignore the old version
@@ -352,11 +349,10 @@ int nvidia_get_card_info(struct nv_ioctl_card_info *ci)
              */
             break;
         default:
-            status = EINVAL;
-            break;
+            return EINVAL;
     }
 
-    /* clear card information structure */
+    ci = args;
     memset(ci, 0, sizeof(ci));
 
     for (i = 0; i < NV_MAX_DEVICES; i++) {
@@ -379,7 +375,7 @@ int nvidia_get_card_info(struct nv_ioctl_card_info *ci)
         ci[i].reg_size       = nv->regs->size;
     }
 
-    return status;
+    return 0;
 }
 
 int nvidia_handle_ioctl(
@@ -430,17 +426,17 @@ int nvidia_handle_ioctl(
 
     switch (nr) {
         case NV_ESC_CHECK_VERSION_STR:
-            status = (rm_perform_version_check(sp, args) == RM_OK)
-                ? 0 : EINVAL;
+            status = ((rm_perform_version_check(sp,
+                            args, size) == RM_OK) ? 0 : EINVAL);
             break;
 
         case NV_ESC_CARD_INFO:
-            status = nvidia_get_card_info(args);
+            status = nvidia_get_card_info(args, size);
             break;
 
         default:
-            status = rm_ioctl(sp, nv, filep, nr, args)
-                ? 0 : EINVAL;
+            status = ((rm_ioctl(sp, nv, filep, nr,
+                            args, size) == RM_OK) ? 0 : EINVAL);
             break;
     }
 
@@ -1159,7 +1155,7 @@ NvS32 nv_alloc_contig_pages(
             attr = VM_MEMATTR_WRITE_BACK;
             break;
         case NV_MEMORY_UNCACHED_WEAK:
-            attr = VM_MEMATTR_UNCACHED;
+            attr = VM_MEMATTR_WEAK_UNCACHEABLE;
             break;
         case NV_MEMORY_WRITECOMBINED:
             attr = VM_MEMATTR_WRITE_COMBINING;
@@ -1298,7 +1294,7 @@ NvS32 nv_alloc_system_pages(
             attr = VM_MEMATTR_WRITE_BACK;
             break;
         case NV_MEMORY_UNCACHED_WEAK:
-            attr = VM_MEMATTR_UNCACHED;
+            attr = VM_MEMATTR_WEAK_UNCACHEABLE;
             break;
         case NV_MEMORY_WRITECOMBINED:
             attr = VM_MEMATTR_WRITE_COMBINING;
@@ -1705,6 +1701,9 @@ int nvidia_mmap_dev_single(
         *offset = (*offset - nv->fb->address);
         return 0;
     } else if (IS_REG_OFFSET(nv, *offset, size)) {
+        if (IS_BLACKLISTED_REG_OFFSET(nv, *offset, size)) {
+            return EINVAL;
+        }
         *object = sc->BAR_objects[NV_GPU_BAR_INDEX_REGS];
         vm_object_reference(*object);
         *offset = (*offset - nv->regs->address);
